/*
 * Created on 27.10.2004
 *
 */
package de.lunqual.rzpro.database;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;

import de.lunqual.rzpro.RzPro;
import de.lunqual.rzpro.log.LogFactory;

/**
 * @author labor
 *
 */
public class DBImport {

    RzPro           		rz;
    Connection      	con;
    Connection		transferConnection;
    DBFactory       	db;
    int					dbStatus;
    Statement			transferWriter;
    Statement			transferReader;
    Statement			rzReader;
    Statement			rzWriter;

    public static final String      DefaultDataBaseDriver = "com.mysql.jdbc.Driver";
    public static final String      DefaultDataBaseURL  = "jdbc:mysql://127.0.0.1:3306/transfer";
    public static final int          DB_CLOSE            = 0;
    public static final int          DB_OPEN             = 1;

    public DBImport(RzPro r, DBFactory aDb, Connection aCon) {
        rz = r;
        db = aDb;
        con = aCon;
        if(dbLogin(DefaultDataBaseDriver,DefaultDataBaseURL,"root","")) {
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.dbopen", "connection established");
            transfer();
        } else {
            dbClose();
        }
    }

    /* dbLogin verbindet mit der Datenbank  */
    public boolean dbLogin(String aDriver,String aUrl,String aUser, String aPwd) {
        boolean ret = false;
        try {
             Class.forName(aDriver);
		}
		catch (final ClassNotFoundException e) {
	            ret = false;
	            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.dbLogin", e.getLocalizedMessage());
	        }
		try {
	            transferConnection = DriverManager.getConnection(aUrl,aUser,aPwd);
	            dbStatus = DB_OPEN;

	            transferWriter=transferConnection.createStatement(ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_UPDATABLE);
	            transferReader=transferConnection.createStatement(ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_READ_ONLY);

	            rzWriter=con.createStatement(ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_UPDATABLE);
	            rzReader=con.createStatement(ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_READ_ONLY);


	            ret = true;
		}
		catch (final SQLException e){
	            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "db.dbimportLogin", e.getLocalizedMessage());
	            ret = false;
	        }
		return ret;
    }



    /** dbClose() schließt die Datenbank  *
     */
    public void dbClose() {
        try {
            if (dbStatus == DB_OPEN){
                transferConnection.close();
                rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.dbClose", "OK");
            }
        }
        catch (final Exception e){
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.dbClose", e.getLocalizedMessage());
        }
    }

    public void transfer() {
        System.out.println("transfer-stub started:");
        transfer_null();
        transfer_adressen();
        transfer_lager();
        transfer_text();
        transfer_convert();
    }

    //vorarbeiten... null-felder entfernen
    private void transfer_null() {
        String sql;
        try {
            System.out.println("Null-Datums-Felder auf sinnvollen Wert setzen > erstellt");
            sql="Update transfer set erstellt='2004-01-01 0:00' where erstellt is null";
            System.out.println(">" + transferWriter.executeUpdate(sql));
            System.out.println("Null-Datums-Felder auf sinnvollen Wert setzen > geaendert");
            sql="Update transfer set geaendert=erstellt where geaendert is null";
            System.out.println(">" + transferWriter.executeUpdate(sql));
            System.out.println("Null-Felder auf sinnvollen Wert setzen > comment");
            sql="Update transfer set text='' where text is null";
            System.out.println(">" + transferWriter.executeUpdate(sql));
            System.out.println("Null-Felder auf sinnvollen Wert setzen > bestellnummer");
            sql="Update transfer set bestellnummer='' where bestellnummer is null";
            System.out.println(">" + transferWriter.executeUpdate(sql));

            System.out.println("währung umstellen >euro");
            sql="Update transfer set waehrung=1 where waehrung=4";
            System.out.println(">" + transferWriter.executeUpdate(sql));

            System.out.println("währung umstellen >dm");
            sql="Update transfer set waehrung=2 where waehrung=5";
            System.out.println(">" + transferWriter.executeUpdate(sql));

        } catch(final Exception e) {
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.transfer_null", e.getLocalizedMessage());
        }
    }

    //stichworte erstellen
    private void transfer_adressen() {
        String sql;
        ResultSet rs;
        try {
            System.out.println("adressen-tabelle-löschen");
            sql="delete from adressen";
            System.out.println(">" + rzWriter.executeUpdate(sql));
            System.out.println("adressen aus musterordner lesen");
            sql="select distinct ordner,ordnerpfad from transfer where ordnerpfad like '%muster%'";
            rs = transferReader.executeQuery(sql);
            while (rs.next()) {
                System.out.print("     write >" + rs.getString("ordner") + " :");
                sql="INSERT INTO adressen (firma) VALUES ('"+rs.getString("ordner") + "')";
                System.out.println(rzWriter.executeUpdate(sql) == 1 ? " ok" : " failed");
            }

            System.out.println("adressen aus aromenordner lesen");
            sql="select distinct ordner,ordnerpfad from transfer where ordnerpfad like '%aromen'";
            rs = transferReader.executeQuery(sql);
            while (rs.next()) {
                System.out.print("     get >" + rs.getString("ordner") + " :");
                if(getAdresse(rs.getString("ordner"))==0) {
                    sql="INSERT INTO adressen (firma) VALUES ('"+rs.getString("ordner") + "')";
                    System.out.println(rzWriter.executeUpdate(sql) == 1 ? " ok" : " failed");
                } else {
                    System.out.println("firma vorhanden");
                }
            }

        } catch(final Exception e) {
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.transfer_adressen", e.getLocalizedMessage());
        }
    }

    private void transfer_lager() {
        String sql;
        try {
            System.out.println("lager erstellen");
            sql="delete from lager";
            System.out.println("delete lager>" + rzWriter.executeUpdate(sql));
            sql="select distinct transfer_Lager_Lager,bezeichnung,liste,ersatz from transfer order by transfer_Lager_Lager";
            final ResultSet rs = transferReader.executeQuery(sql);
            while (rs.next()) {
                System.out.println("Lager>"+rs.getString("bezeichnung"));
                sql="insert into lager (id,name,liste,ersatztext) VALUES (" + String.valueOf(rs.getInt("transfer_Lager_Lager")+1)   +",'" + rs.getString("bezeichnung") + "'," +
                	String.valueOf(rs.getInt("liste")) + ",'" + rs.getString("ersatz") + "')";
                System.out.println(rzWriter.executeUpdate(sql) != 0 ? " insert ok " : " insert failed");
            }
        } catch(final Exception e) {
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.transfer_lager", e.getLocalizedMessage());
        }
    }
    private int getAdresse(String adr) {
        int ret = 0;
        try {
            final Statement reader=con.createStatement(ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_READ_ONLY);
            System.out.print("     get adresse:"+adr + "  =");
            final String sql = "select id from adressen where firma='"+adr+"'";
            final ResultSet rs = reader.executeQuery(sql);
            if(rs.next()) {
                ret = rs.getInt("id");
                System.out.println(ret);
            }
        }catch (final Exception e) {
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.getAdresse", e.getLocalizedMessage());
            ret = 0;
            System.out.println(" nicht gefunden");
        }
        return ret;
    }

    private void transfer_convert() {
        String sql;
        try {
            String out;
            String in;
            int id;
            System.out.println("rezepturen konvertieren");
            sql="select id,typ,name,comment from rezeptliste";
            final ResultSet rs = rzReader.executeQuery(sql);
            while (rs.next()) {
                System.out.println("convert >"+rs.getInt("id") + " >" + rs.getString("name") + "  :");
                in=rs.getString("comment");
                id=rs.getInt("id");
                if(rs.getInt("typ") == RzPro.TYP_ID_REZEPT) {
	                if(in.indexOf("[[") == -1) {
	                    out = CommentToRaw(in);
	                    if(out != null) {
	                        sql = "update rezeptliste set comment='" + out + "' where id=" + String.valueOf(id);
	                        System.out.println(rzWriter.executeUpdate(sql) != 0 ? "  write ok " : " write failed");
	                    }
	                }
                }
            }
        } catch(final Exception e) {
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.transfer_convert", e.getLocalizedMessage());
        }
    }

    public void transfer_text(){
    	System.out.println("transfer-text");
    	int anzahl=0;
    	try {
    	    final Statement read=transferConnection.createStatement(ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_READ_ONLY);
    	    final PreparedStatement write=con.prepareStatement("INSERT INTO rezeptliste " +
    	    		"(erstellt,geaendert,name,typ,comment,staerke,litergewicht,extrakt,extraktfrei,stichworte,adressen,artikelnummer," +
    	    		"basismenge,basiseinheit,preismenge,preis,preiseinheit,waehrung,bestandseinheit,lager,schwundsatz)"+
    	    		" VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?) " +
    	    		"",ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_UPDATABLE);



    	    final ResultSet reader = read.executeQuery("SELECT * FROM transfer");
    	    System.out.println("rezeptliste löschen");
    	    System.out.println(">" + write.executeUpdate("delete from rezeptliste"));
       	    System.out.println("nicht gefunden>" + write.executeUpdate("insert into rezeptliste (id,name) values (1,'nicht gefunden')"));

    	    String name;
    	    int typ;
    	    int adresse;
    	    String tmp;
    	    String tmp1;
    	    String comment;
    	    Date erstellt = null;
    	    Date geaendert = null;
    	    double staerke;
    	    double litergewicht;
    	    boolean extraktfrei;
    	    double extrakt;
    	    double schwund;
    	    String stichworte="";
    	    String adressen="";
    	    String artikelnummer="";
    	    boolean isNotiz;
    	    double preismenge=0;
    	    BigDecimal preis = new BigDecimal(0.0);
    	    int preiseinheit=0;
    	    double basismenge=0;
    	    int basiseinheit=0;
    	    int waehrung = 0;
    	    int bestandseinheit=0;
    	    int lager=0;
    	    while (reader.next()) {
    	        anzahl++;
    	        //-----------------------titel---------------------------
    	        name = reader.getString("titel").trim();
    	        //-----------------------comment---------------------
    	        comment = reader.getString("text").trim();
    	        //-------------------erstellt---------------------------
    	        erstellt = db.getDateFromString(reader.getString("erstellt"));
    	        //-------------------geaendert---------------------------
    	        geaendert = db.getDateFromString(reader.getString("geaendert"));
    	        //--------------------typ--------------------------------
    	        if(reader.getString("tag").trim().equals("#n")) {
    	            stichworte = "<2>";
    	            typ = RzPro.TYP_ID_NOTIZ;
    	            isNotiz=true;
    	        } else {
    	            stichworte = "<1>";
    	            typ= RzPro.TYP_ID_REZEPT;
    	            isNotiz=false;
    	        }
    	        if(!isNotiz) {
	    	        //--------------staerke-----------------
	    	        staerke = reader.getDouble("staerke");
	    	        //-------------litergewicht-------------------
	    	        litergewicht = reader.getDouble("gewicht");
	    	        //----------------------extraktfrei--------------------
	    	        extraktfrei = reader.getBoolean("autogewicht");
	    	        //---------------------extrakt------------------------
	    	        extrakt = reader.getDouble("trockensubstanz");
	    	        //artikelnummer
	    	        artikelnummer=reader.getString("bestellnummer").trim();
	    	        //basismenge
	    	        basismenge = reader.getDouble("basis");
	    	        // basiseinheit
	    	        basiseinheit= reader.getInt("einheit");
	    	        // preis
	    	        preis = new BigDecimal(reader.getDouble("preis"));
	    	        //preismenge
	    	        preismenge = reader.getDouble("preismenge");
	    	        //preiseinheit
	    	        preiseinheit = reader.getInt("preiseinheit");
	    	        //währung
	    	        waehrung=reader.getInt("waehrung");
	    	        //bestandseinheit
	    	        bestandseinheit=reader.getInt("bestandeinheit");
	    	        if (staerke != 0) {
	    	            bestandseinheit=3; // alle alkoholhaltigen zutaten in LA führen
	    	        }
	    	        schwund = reader.getDouble("schwund");
	    	        //lager
	    	        // muss um eins erhöht werden
	    	        lager = reader.getInt("rezeptliste_Lager") + 1;
    	        } else {
    	            staerke=0;
    	            litergewicht=0.998201;
    	            extraktfrei=true;
    	            extrakt=0;
    	            artikelnummer="";
    	            basismenge=0;
    	            basiseinheit=0;
    	            preis=new BigDecimal(0.0);
    	            preiseinheit=0;
    	            preismenge=0;
    	            waehrung=0;
    	            lager=0;
    	            schwund=0;
    	        }
    	        //stichwortezuordnen
    	        tmp=reader.getString("ordnerpfad");
    	        tmp1 = reader.getString("ordner");
    	        if(tmp.toLowerCase().indexOf("muster") != -1) {
    	            stichworte += "<3>";
    	        }
    	        if((tmp.toLowerCase().indexOf("aromen") != -1) || (tmp1.toLowerCase().indexOf("aromen") != -1)) {
    	            stichworte += "<5>";
    	        }
    	        if((tmp.toLowerCase().indexOf("likör") != -1) || (tmp1.toLowerCase().indexOf("likör") != -1)) {
    	            stichworte += "<6>";
    	        }
    	        if((tmp.toLowerCase().indexOf("zucker") != -1) || (tmp1.toLowerCase().indexOf("zucker") != -1)) {
    	            stichworte += "<7>";
    	        }
    	        if((tmp.toLowerCase().indexOf("rohwaren") != -1) || (tmp1.toLowerCase().indexOf("rohwaren") != -1)) {
    	            stichworte += "<8>";
    	        }
    	        if((tmp.toLowerCase().indexOf("zutaten") != -1) || (tmp1.toLowerCase().indexOf("zutaten") != -1)) {
    	            stichworte += "<9>";
    	        }
    	        if(!rz.isZero(extrakt)) {
    	            stichworte += "<10>";
    	        }
    	        if(reader.getString("bezeichnung").toLowerCase().indexOf("versteuert") != -1) {
    	            stichworte += "<11>";
    	        }
    	        if(reader.getString("bezeichnung").toLowerCase().indexOf("fertigwaren") != -1) {
    	            stichworte += "<12>";
    	        }
    	        if((tmp.toLowerCase().indexOf("drogen") != -1) || (tmp1.toLowerCase().indexOf("drogen") != -1)) {
    	            stichworte += "<13>";
    	        }
    	        if((tmp.toLowerCase().indexOf("labor") != -1) || (tmp1.toLowerCase().indexOf("labor") != -1)) {
    	            stichworte += "<14>";
    	        }
    	        //adressen
    	        adresse = getAdresse(reader.getString("ordner"));
    	        if(adresse != 0) {
    	            adressen = "<" + String.valueOf(adresse)+">";
    	        } else {
    	            adressen="";
    	        }

    	        System.out.print("----------------------------------------\nAnzahl="+anzahl+"\n");
    	        System.out.println("titel="+name);
    	        System.out.println("stichworte="+stichworte);
    	        System.out.println("adressen="+adressen);
    	        System.out.println("------------------------------------------");

    	        write.setString(1,db.dbGetDateFormatString( erstellt));
    	        write.setString(2,db.dbGetDateFormatString(geaendert));
    	        write.setString(3,name);
    	        write.setInt(4,typ);
    	        write.setString(5,comment);
    	        write.setDouble(6,staerke);
    	        write.setDouble(7,litergewicht);
    	        write.setDouble(8,extrakt);
    	        write.setBoolean(9,extraktfrei);
    	        write.setString(10,stichworte);
    	        write.setString(11,adressen);
    	        write.setString(12, artikelnummer);
    	        write.setDouble(13,basismenge);
    	        write.setInt(14,basiseinheit);
    	        write.setDouble(15, preismenge);
    	        write.setBigDecimal(16,preis);
    	        write.setInt(17, preiseinheit);
    	        write.setInt(18,waehrung);
    	        write.setInt(19,bestandseinheit);
    	        write.setInt(20,lager);
    	        write.setDouble(21,schwund);
    	        System.out.println(write.executeUpdate() != 0 ? " write ok " : " write failed");
    	    }
    	} catch(final Exception e) {
            rz.getLogFactory().logMessage(LogFactory.LOG_WARN, "dbimport.transfer", e.getLocalizedMessage());
    	}
    }

    /** boolean isRechenzeile(String) prüft ob die vorliegende Zeile eine
     * Rechen/Sternzeile ist oder nicht
     * Es wird _nicht_ geprüft, ob eine angegebene Zutat in der
     * Datenbank existiert !!!
     */
    public boolean isRechenzeile(String aString){
        boolean ret = false;
        if (!aString.equals("")){
            final String tokens[] = aString.split(" ");
            switch (tokens.length){
                case 0:
                case 1:
                case 2: // Zeilen mit 0,1 oder 2 Elementen können keine
                        // Rechenzeilen sein...
                    ret = false;
                    break;
                default:// Zeilen mit 3 oder mehr Elementen _können_
                        // Rechenzeilen sein
                    if(rz.getLocale().isNumber(tokens[0].trim())){
                        if(rz.getDatabase().getEinheiten().isEinheit(tokens[1].trim())){
                        	ret = true;
                        }
                    }
            }
        }
        return ret;
    }

    /** sichtbaren comment in Raw umwandeln
     * gibt den String zurück, der in die Datenbank gesichert wird
     * raw = Datenbankfeld-Inhalt
     * comment = Textfeld-Inhalt
     * @param aComment der KommentarString
     */
    public String CommentToRaw(String aComment){
        String raw = "";
        String raw_2;
        String zeilen[];
        String items[];
        int dbKey;
        int i;
        int anfang,ende;
        boolean exclusive;
        zeilen = aComment.split("\n");
        for (i=0;i < zeilen.length;i++){
            //Zeile für zeile die Art der zeile identifizieren
            if(isRechenzeile(zeilen[i].trim())){
                // Rechenzeile
            	zeilen[i] = zeilen[i].trim();
            	raw_2 = zeilen[i].replaceAll("\\\\s+", " ");
                raw += RzPro.TYP_CALC + " ";
                items = raw_2.trim().split(" ");
                if(rz.getLocale().isNumber(items[0].trim())){
                    // doppelt hält besser !
                    raw += items[0].trim() + " ";
                } else {raw += "0 ";}
                if(rz.getDatabase().getEinheiten().isEinheit(items[1].trim())){
                    raw += rz.getDatabase().getEinheiten().getEinheit(items[1].trim()).getID() + " ";
                }else {raw += "0 ";}
                ende = -1;
                anfang = zeilen[i].indexOf(items[2]);
                try {
                    ende = zeilen[i].substring(anfang).indexOf("[[");
                }catch (final Exception e){}
                if(ende != -1){
					dbKey = Integer.parseInt(zeilen[i].substring(anfang+ende+2,zeilen[i].indexOf("]]",anfang+ende)));
                	exclusive = false;
                }
                else {
                    dbKey = rz.getDatabase().getRezeptur().dbGetRezepturID(zeilen[i].substring(anfang,zeilen[i].length()).trim());
                    exclusive = true;
                }
                if (dbKey != 0){
                    raw += "[[" + String.valueOf(dbKey) + "]] ";
                    if (!exclusive) {
						raw += "[[" + dbKey + "]] ";
					}
                }
                else{
                	raw += "[[1]]\n0 >>"+zeilen[i].substring(anfang,zeilen[i].length()) + "<<";
                }
                raw += "\n";
            }
            else {
                // Kommentar
                raw += RzPro.TYP_COMMENT + " " + zeilen[i].trim() + "\n";
            }
        }
        if(raw.equals("\n")) {
			raw = "";
		}
        return raw;
    }
}
